/*
 *  Wifi testing functions
 *  Perform 3 testing cases mentioned in wifi organization using Chariot NetIQ
 *    1) File Send long
 *    2) Inquery Long
 *    3) Real Audio Streaming
 *  Copyright (c) by Alamy Liu <alamyliu@yahoo.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#include	<signal.h>
#include	<syslog.h>
#include	<time.h>

#include	"ttcp.h"
#include	"wifi.h"

/* file long testing case matches to ttcp's design */
#define	Wifi_transfer_file_long		Transfer_data
#define	Wifi_receive_file_long		Receive_data

extern	MemoryResource	memRes;

#define	WIFI2_OUTPUT_FORMAT \
		"%9bs%9br%8.3rt%12.4rbr%8f"

void
Wifi_transfer_inquery_long(int fd, Options *opt)
{
int		loop;
int		n;
int		cntWrite, cntRead;
BenchmarkData	*bench;

	for (loop = 0; loop < opt->Loop; loop++)
	{
	bench = memRes.benchmarkArray + loop;
	n = opt->BufCnt;

	bench->nRecvBytes = 0;
	bench->nSendBytes = 0;
	bench->nBuffer = 0;
	signal(SIGPIPE, SigPipe);
	StartTimer(NULL);
	while (n--) {

		cntWrite = Nwrite(fd, memRes.buf, opt->BufLen);
		cntRead = Nread(fd, memRes.buf, opt->BufLen);
		bench->nSendBytes += opt->BufLen;
		bench->nRecvBytes += opt->BufLen;
		bench->nBuffer++;
	}
	StopTimer(NULL, &bench->cput, &bench->realt);

	}	// end of for(loop)
}

void
Wifi_receive_inquery_long(int fd, Options *opt)
{
int		loop;
int		n;
int		cntWrite, cntRead;
BenchmarkData	*bench;

	for (loop = 0; loop < opt->Loop; loop++)
	{
	bench = memRes.benchmarkArray + loop;
	n = opt->BufCnt;

	bench->nRecvBytes = 0;
	bench->nSendBytes = 0;
	bench->nBuffer = 0;
	StartTimer(NULL);
	while (n--) {
		cntRead = Nread(fd, memRes.buf, opt->BufLen);
		cntWrite = Nwrite(fd, memRes.buf, opt->BufLen);
		bench->nRecvBytes += opt->BufLen;
		bench->nSendBytes += opt->BufLen;
		bench->nBuffer++;
	}
	StopTimer(NULL, &bench->cput, &bench->realt);

	}	// end of for(loop)
}

/*
 * Throughput (rate) = Bytes(transfered) / Time(Transfer + Delay)
 *   Time (Transfer + Delay) = Bytes / Rate
 *   Time(Delay) = (Bytes transfered / Rate) - Time(Transfer)
 */
void
Wifi_transfer_real_audio(int fd, Options *opt)
{
int		loop;
int		n;
int		cntWrite, cntRead;
BenchmarkData	*bench;
double	rate,			// Transfer rate (throughput)
		trans_cput,		// We don't use thie value, just pass to function
		trans_realt,	// transfer time : Time(Transfer)
		trans_delay,	// How long should we delay : Time(Delay)
		try_delay,		// Hope to delay [try_delay] seconds
		delay_realt;	// The real time delayed
Timer	delta_timer;
//struct timespec	sleep_time, remain_time;
int		finish_delay;	// a flag to control delay loop

	rate = 10334.208;	// 80.736 kbps = 10334.208 Bytes/s = 0.07884375 MBit/s
//	rate = 10092.0;		// usleep() may be lengthened slightly,
						// so I set higher rate

	for (loop = 0; loop < opt->Loop; loop++)
	{
	bench = memRes.benchmarkArray + loop;
	n = opt->BufCnt;

	bench->nRecvBytes = 0;
	bench->nSendBytes = 0;
	bench->nBuffer = 0;
	signal(SIGPIPE, SigPipe);
	StartTimer(NULL);
	while (n--) {
//		StartTimer( &delta_timer );
		cntWrite = Nwrite(fd, memRes.buf, opt->BufLen);
		StopTimer(NULL, &trans_cput, &trans_realt);
		
		bench->nSendBytes += opt->BufLen;
		bench->nBuffer++;

		/*
		 * [bench->nSendBytes] bytes were transferred in [delta_realt] seconds.
		 * Should delay ((opt->BufLen / rate) - trans_realt) seconds
		 */
		trans_delay = (bench->nSendBytes / rate) - trans_realt;
		if (trans_delay > (double)0.0) {
			unsigned long	try_delay;

			StartTimer( &delta_timer );

			delay_realt = (double)0.0;
			while (delay_realt < trans_delay)
			{
				try_delay = trans_delay - delay_realt;
				usleep( (unsigned long) try_delay * 1000000 );

//				for (finish_delay = 0; finish_delay < 100000; finish_delay++);

				StopTimer( &delta_timer, &trans_cput, &delay_realt);
			}	// end of for(finish)
		} else {
			// We are too slow ...
		}

/*
syslog(LOG_DEBUG, "Send %d/%f, should delay %f/%f\n",
		opt->BufLen, trans_realt,
		trans_delay, delay_realt);
*/

	}
	StopTimer(NULL, &bench->cput, &bench->realt);

	}	// end of for(loop)
}

void
Wifi_receive_real_audio(int fd, Options *opt)
{
int		loop;
int		n;
int		cntWrite, cntRead;
BenchmarkData	*bench;

	for (loop = 0; loop < opt->Loop; loop++)
	{
	bench = memRes.benchmarkArray + loop;
	n = opt->BufCnt;

	bench->nRecvBytes = 0;
	bench->nSendBytes = 0;
	bench->nBuffer = 0;
	StartTimer(NULL);
	while (n--) {
		cntRead = Nread(fd, memRes.buf, opt->BufLen);
		bench->nRecvBytes += opt->BufLen;
		bench->nBuffer++;
	}
	StopTimer(NULL, &bench->cput, &bench->realt);

	}	// end of for(loop)
}

void
Wifi_transfer_data(int fd, Options *opt)
{
	switch (opt->Wifi)
	{
	case 1:	Wifi_transfer_file_long(fd, opt);
			break;
	case 2:	Wifi_transfer_inquery_long(fd, opt);
			break;
	case 3:	Wifi_transfer_real_audio(fd, opt);
			break;
	default:		/* Original testing method */
			Transfer_data(fd, opt);
			break;
	}
}

void
Wifi_receive_data(int fd, Options *opt)
{
	switch (opt->Wifi)
	{
	case 1:	Wifi_receive_file_long(fd, opt);
			break;
	case 2:	Wifi_receive_inquery_long(fd, opt);
			break;
	case 3:	Wifi_receive_real_audio(fd, opt);
			break;
	default:		/* Original testing method */
			Receive_data(fd, opt);
			break;
	}
}

void
Wifi_set_options(Options *opt)
{
	switch (opt->Wifi)
	{
	case 1:
			break;
	case 2:
			opt->BufCnt = 50;
			if (opt->BufLen > 100) {
				opt->BufLen = 100;
			}
			opt->OutputFmt = WIFI2_OUTPUT_FORMAT;
			break;
	case 3:
			opt->udp = 1;
			opt->BufCnt = 40;
			if (opt->BufLen > 351) {
				opt->BufLen = 351;
			}
			break;
	default:
			break;
	}
}
